require 'tmpdir'

require 'static_mock/version'
require 'static_mock/step'
require 'static_mock/object_extension'

module StaticMock
  @@steps = []
  @@mocks = []

  OUTPUT_DIR = File.join(Dir.tmpdir, '/static_mock')

  def self.init
    Object.include(ObjectExtension)

    Dir.mkdir(OUTPUT_DIR) unless Dir.exist?(OUTPUT_DIR)

    @@steps = []
    @@mocks = []
  end

  def self.define_step(step_name, &block)
    raise StepDefinitionError if block.nil?

    step_out_file = File.join(OUTPUT_DIR, "#{step_name}.yml")
    File.delete(step_out_file) if File.exist?(step_out_file)

    Step.new(step_name, block).tap { |step| @@steps << step }
  end

  def self.run
    @@steps.each do |step|
      step.run

      @@mocks.each { |o| o.static_mock(step.step_name, OUTPUT_DIR) }
    end
  end

  def self.mock(*mock_list)
    @@mocks = mock_list
  end

  def self.load_mocks(step_name)
    ret = []
    $/="\n\n"
    File.open(File.join(OUTPUT_DIR, "#{step_name}.yml"), 'r').each do |obj|
      ret << YAML::load(obj)
    end
    ret
  end

  class StepDefinitionError < StandardError
    def message
      "Step definition requires a block! Please provide one."
    end
  end
end
